// Copyright 2017 Budapest University of Technology and Economics (BME IK)
//
// This file is part of CIRCLE Cloud.
//
// CIRCLE is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// CIRCLE is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along
// with CIRCLE.  If not, see <http://www.gnu.org/licenses/>.

#ifndef OCCILIB_STORAGE_H_
#define OCCILIB_STORAGE_H_

#include <string>

#include "json.hpp"
#include "resource.h"

namespace OcciClient {

    class Storage : public Resource {

        private:

            double size; // in GB
            std::string state;
            std::string stateMessage;

            void parseJsonInstance(nlohmann::json instance) {
                Resource::parseJsonInstance(instance);
                this->state = getAttributeFromRendering<std::string>(instance, "occi.storage.state");
                this->stateMessage = getAttributeFromRendering<std::string>(instance, "occi.storage.state.message");
                this->size = getAttributeFromRendering<double>(instance, "occi.storage.size");
                this->rendering = instance;
            }

            Storage(OcciSession* session, nlohmann::json instance)
                : Resource(session, "storage", instance) {
                parseJsonInstance(instance);
            }

            void invokeAction(nlohmann::json actionData) {
                auto inst = Resource::invokeAction(actionData);
                parseJsonInstance(inst);
            }

        public:

            static std::shared_ptr<Storage> getStorageInstance(OcciSession* session, std::string id) {
                auto uri = "storage/" + id + "/";
                auto instance = session->get(uri);
                return std::make_shared<Storage>(Storage(session, instance));
            }

            std::string getState(){
                return this->state;
            }

            std::string getStateMessage(){
                return this->stateMessage;
            }

            double getSize(){
                return this->size;
            }

    };

}

#endif // OCCILIB_STORAGE_H
